package org.mapsforge.android.maps;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.tipple.mapDisplay.R;
import org.tipple.mapDisplay.TakePicture;
import org.tipple.mapDisplay.tippleActivity;

import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.graphics.Point;
import android.graphics.Rect;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.MeasureSpec;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout.LayoutParams;
import android.widget.TextView;

/**
 * Thread-safe implementation of the {@link ItemizedMarkerOverlay} class using
 * an {@link ArrayList} as internal data structure. casey: implement the ontap
 * method for the customized overlay item
 */
public class ArrayItemizedMarkerOverlay extends
		ItemizedMarkerOverlay<OverlayMarkerItem> {
	private static final int ARRAY_LIST_INITIAL_CAPACITY = 8;
	private static final String THREAD_NAME = "ArrayItemizedMarkerOverlay";

	protected final Context context;
	protected AlertDialog.Builder builder;// Builder builder
	protected View dialogLayout;
	protected AlertDialog alertDialog;
	Bitmap bmScreen;

	protected OverlayItem item;
	protected final ArrayList<OverlayMarkerItem> overlayMarkerItems;
	protected OverlayMarkerItem overlayItem;
	protected Drawable marker;
	protected Drawable defaultMarker;

	// casey
	private List<Integer> visibleItems;
	private int numberOfSize;
	// private OverlayMarkerItem checkOverlayItem;
	// casey: popup window
	private MarkerPopup popup;
	private String imageName;

	/**
	 * Constructs a new ArrayItemizedOverlay.
	 * 
	 * @param defaultMarker
	 *            the default marker.
	 * @param context
	 *            the reference to the application context.
	 */
	public ArrayItemizedMarkerOverlay(Drawable defaultMarker, Context context) {

		super(boundCenterBottom(defaultMarker));
		this.context = context;
		this.defaultMarker = defaultMarker;
		this.overlayMarkerItems = new ArrayList<OverlayMarkerItem>(
				ARRAY_LIST_INITIAL_CAPACITY);
		this.visibleItems = new ArrayList<Integer>(numberOfSize);

	}

	public void setMarker(Drawable marker) {
		this.marker = boundCenterBottom(marker);
	}

	public Drawable getMarker() {
		return this.marker;
	}

	/**
	 * Adds the given item to the Overlay.
	 * 
	 * @param overlayItem
	 *            the item that should be added to the Overlay.
	 */
	public synchronized void addOverlay(OverlayMarkerItem overlayItem) {
		this.overlayMarkerItems.add(overlayItem);
		populate();
	}

	/**
	 * Adds all items of the given collection to the Overlay.
	 * 
	 * @param c
	 *            collection whose items should be added to the Overlay.
	 */
	public synchronized void addOverlays(
			Collection<? extends OverlayMarkerItem> c) {
		this.overlayMarkerItems.addAll(c);
		populate();
	}

	/**
	 * Removes all items from the Overlay.
	 */
	public synchronized void clear() {
		this.overlayMarkerItems.clear();
		populate();
	}

	@Override
	public String getThreadName() {
		return THREAD_NAME;
	}

	/**
	 * Removes the given item from the Overlay.
	 * 
	 * @param overlayMarkerItem
	 *            the item that should be removed from the Overlay.
	 */
	public synchronized void removeOverlay(OverlayMarkerItem overlayMarkerItem) {
		this.overlayMarkerItems.remove(overlayMarkerItem);
		populate();
	}

	@Override
	public synchronized int size() {
		numberOfSize = this.overlayMarkerItems.size();
		return this.overlayMarkerItems.size();
	}

	@Override
	public synchronized OverlayMarkerItem createItem(int i) {
		return this.overlayMarkerItems.get(i);
	}

	protected boolean onTap(int index, MapView mapView) {

		this.overlayItem = createItem(index);
		if (this.overlayItem != null) {
			if (this.overlayItem.getDescription() != null) {
				hideOtherDialog(this.overlayMarkerItems, this.overlayItem,
						mapView);
				LayoutInflater inflater = (LayoutInflater) this.context
						.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				View view = inflater.inflate(R.layout.custom_dialog, null);
				dialogLayout = (View) view.findViewById(R.id.ScrollView01);// ScrollView01
				TextView text = (TextView) dialogLayout.findViewById(R.id.text);
				text.setText(this.overlayItem.getDescription() + "\n" + "\n"
						+ this.overlayItem.getContent());
				// String fileName = "/sdcard/";
				String fileName = tippleActivity.imageDirectory;
				fileName += this.overlayItem.getDescription() + ".jpg";
				File imgFile = new File(fileName);

				// Take image for chapter without image initially
				// save the image then next time can retrieve it
				if (!imgFile.exists()) {
					Button camera = (Button) dialogLayout
							.findViewById(R.id.button);
					camera.setEnabled(true);
					camera.setVisibility(0);
					String tmpFileName = this.overlayItem.getDescription()
							+ ".jpg";
					camera.setOnClickListener(new RandomClickListener(
							tmpFileName));
				}

				if (imgFile.exists()) {

					Bitmap bitmap = BitmapFactory.decodeFile(imgFile
							.getAbsolutePath());
					int width = bitmap.getWidth();
					int height = bitmap.getHeight();
					int newWidth = 200;
					int newHeight = 150;
					float scaleWidth = ((float) newWidth) / width;
					float scaleHeight = ((float) newHeight) / height;
					Matrix matrix = new Matrix();
					matrix.postScale(scaleWidth, scaleHeight);

					Bitmap resizedBitmap = Bitmap.createBitmap(bitmap, 0, 0,
							width, height, matrix, true);
					BitmapDrawable bmd = new BitmapDrawable(resizedBitmap);
					ImageView image = (ImageView) dialogLayout
							.findViewById(R.id.image);
					image.setImageDrawable(bmd);

				}

				builder = new AlertDialog.Builder(this.context);
				builder.setView(dialogLayout);

				builder.setPositiveButton("OK", null);
				alertDialog = builder.create();
				alertDialog.show();
			}
			return true;
		}
		return false;

	}

	public boolean onTap(GeoPoint p, MapView mapView) {

		Projection projection = mapView.getProjection();
		Point eventPosition = projection.toPixels(p, null);

		// check if the translation to pixel coordinates has failed
		if (eventPosition == null) {
			return false;
		}
		OverlayMarkerItem checkOverlayItem;
		Point checkItemPoint = new Point();
		Rect checkMarkerBounds;
		int checkLeft;
		int checkRight;
		int checkTop;
		int checkBottom;

		synchronized (this.visibleItems) {
			int numberOfSize = this.size();
			Context contexto = mapView.getContext();
			if (numberOfSize < 1) {
				return false;
			}

			for (int i = 0; i < this.size(); ++i) {
				checkOverlayItem = createItem(i);

				if (checkOverlayItem == null) {
					continue;
				}
				synchronized (checkOverlayItem) {
					if (checkOverlayItem.getPoint() == null) {
						continue;
					}

					checkItemPoint = projection.toPixels(
							checkOverlayItem.getPoint(), checkItemPoint);
					// check if the translation to pixel coordinates has failed
					if (checkItemPoint == null) {
						continue;
					}
					checkMarkerBounds = this.defaultMarker.getBounds();

					checkLeft = checkItemPoint.x + checkMarkerBounds.left;
					checkRight = checkItemPoint.x + checkMarkerBounds.right;
					checkTop = checkItemPoint.y + checkMarkerBounds.top;
					checkBottom = checkItemPoint.y + checkMarkerBounds.bottom;

					if (checkRight >= eventPosition.x
							&& checkLeft <= eventPosition.x
							&& checkBottom >= eventPosition.y
							&& checkTop <= eventPosition.y) {

						return onTap(i, mapView);
					}
				}// end of synchronized (checkOverlayItem)
			}
		}

		// no hit
		return false;
	}

	private boolean onBalloonTap(MapView mapView,
			OverlayMarkerItem checkOverlayItem) {
		hideOtherBalloons(this.overlayMarkerItems, checkOverlayItem);
		this.popup = new MarkerPopup(mapView);
		MapView.LayoutParams params = new MapView.LayoutParams(
				LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
		popup.setData(checkOverlayItem);

		// position
		GeoPoint point = checkOverlayItem.getPoint();

		// change parameter
		popup.setLayoutParams(params);

		// visibility
		popup.setVisibility(View.VISIBLE);
		mapView.addView(popup, params);

		popup.setDrawingCacheEnabled(true);
		popup.measure(MeasureSpec.makeMeasureSpec(0, MeasureSpec.UNSPECIFIED),
				MeasureSpec.makeMeasureSpec(0, MeasureSpec.UNSPECIFIED));
		popup.layout(0, 0, popup.getMeasuredWidth(), popup.getMeasuredHeight());
		popup.buildDrawingCache(true);
		popup.setDrawingCacheEnabled(false); // clear drawing
												// cache

		mapView.getController().setCenter(point);

		return true;

	}

	public void hideDialog(MapView mapView) {
		if (dialogLayout != null) {
			mapView.removeView(dialogLayout);
		}

	}

	private void hideOtherDialog(ArrayList<OverlayMarkerItem> overlays,
			OverlayMarkerItem item, MapView mapView) {

		for (OverlayMarkerItem overlay : overlays) {
			if ((overlay != null) && (overlay != item))
				hideDialog(mapView);
		}

	}

	public void hideBalloon() {
		if (popup != null) {
			popup.setVisibility(View.GONE);
		}

	}

	private void hideOtherBalloons(ArrayList<OverlayMarkerItem> overlays,
			OverlayMarkerItem item) {

		for (OverlayMarkerItem overlay : overlays) {
			if ((overlay != null) && (overlay != item))
				hideBalloon();
		}

	}

	public synchronized boolean unTap(GeoPoint p, MapView mapView) {

		alertDialog.cancel();
		return true;
	}

	// inner class for taking pictures
	public class RandomClickListener implements View.OnClickListener {
		private final String tmpImage;

		public RandomClickListener(final String randomIndex) {
			this.tmpImage = randomIndex;
		}

		@Override
		public void onClick(View view) {
			Intent myIntent = new Intent(view.getContext(), TakePicture.class);
			myIntent.putExtra("filename", tmpImage);
			context.startActivity(myIntent);

		}
	}

}