package org.mapsforge.android.maps;

import android.graphics.Point;
import android.graphics.drawable.Drawable;

/**
 * An OverlayMarkerItem holds all parameters of a single element on an
 * {@link ItemizedMarkerOverlay}.
 */
public class OverlayMarkerItem {
	/**
	 * Marker used to indicate the item.
	 */
	protected Drawable marker;

	/**
	 * Geographical position of the item.
	 */
	protected final GeoPoint point;

	/**
	 * Title of the item.
	 */
	protected final String title;

	/*
	 * description of the itme
	 */
	protected final String description;
	protected final String content;
	/**
	 * 
	 * Cached position of the item on the map.
	 */
	Point cachedMapPosition;

	/**
	 * Zoom level of the cached map position.
	 */
	byte cachedZoomLevel;

	/**
	 * Constructs a new OverlayItem.
	 * 
	 * @param point
	 *            the geographical position of the item.
	 */
	public OverlayMarkerItem(GeoPoint point, String title, Drawable balloon) {
		this.point = point;
		this.title = title;
		this.marker = balloon;
		this.description = null;
		this.cachedZoomLevel = Byte.MIN_VALUE;
		this.content = null;
	}

	public OverlayMarkerItem(GeoPoint point, String title) {
		this.point = point;
		this.title = title;
		this.description = null;
		this.cachedZoomLevel = Byte.MIN_VALUE;
		this.content = null;
	}

	public OverlayMarkerItem(GeoPoint point, String title, String description,
			String content) {
		this.point = point;
		this.title = title;
		this.description = description;
		this.content = content;
	}

	protected static Drawable boundCenter(Drawable balloon) {
		balloon.setBounds(balloon.getIntrinsicWidth() / -2,
				balloon.getIntrinsicHeight() / -2,
				balloon.getIntrinsicWidth() / 2,
				balloon.getIntrinsicHeight() / 2);
		return balloon;
	}

	/**
	 * Sets the bounds of the given drawable so that (0,0) is the center of the
	 * bounding box.
	 * 
	 * @param balloon
	 *            the drawable whose bounds should be set.
	 * @return the given drawable.
	 */
	protected static Drawable boundCenterBottom(Drawable balloon) {
		balloon.setBounds(balloon.getIntrinsicWidth() / -2,
				-balloon.getIntrinsicHeight(), balloon.getIntrinsicWidth() / 2,
				0);
		return balloon;
	}

	/**
	 * Returns the marker used to indicate the item (may be null).
	 * 
	 * @return the marker used to indicate the item.
	 */
	public Drawable getMarker() {
		return this.marker;
	}

	/**
	 * Returns the position of the item.
	 * 
	 * @return the position of the item.
	 */
	public GeoPoint getPoint() {
		return this.point;
	}

	/**
	 * Returns the title of the item.
	 * 
	 * @return the title of the item.
	 */
	public String getTitle() {
		return this.title;
	}

	/**
	 * Return the description of the item
	 */
	public String getDescription() {
		return this.description;
	}

	public String getContent() {
		return this.content;
	}

	/**
	 * Sets the marker that is drawn on the map for this item. If the item
	 * marker is null, the default marker will be drawn instead.
	 * 
	 * @param marker
	 *            the marker that is drawn on the map for this item.
	 */
	public void setMarker(Drawable marker) {
		this.marker = boundCenterBottom(marker);
	}

}
